SystemOrganization addCategory: #'Seaside-Session'!
SystemOrganization addCategory: #'Seaside-Session-Filter'!

!WARequestContext methodsFor: '*seaside-session' stamp: 'jf 7/16/2008 10:49'!
application
	"Answer the current application or nil."

	^ handlers 
		detect: [ :each | each isApplication ]
		ifNone: [ nil ]! !

!WARequestContext methodsFor: '*seaside-session' stamp: 'lr 2/20/2009 11:05'!
redirectTo: aUrlOrString 
	self responseGenerator
		redirectTo: aUrlOrString;
		respond! !

!WARequestContext methodsFor: '*seaside-session' stamp: 'jf 7/25/2008 20:34'!
session
	"Answer the current session or nil."

	^ handlers 
		detect: [ :each | each isSession ]
		ifNone: [ nil ]! !

!WAAnchorTag methodsFor: '*seaside-session' stamp: 'lr 8/17/2007 15:08'!
document: aDocument
	self document: aDocument mimeType: nil! !

!WAAnchorTag methodsFor: '*seaside-session' stamp: 'lr 8/17/2007 15:08'!
document: aDocument mimeType: mimeType
	self document: aDocument mimeType: mimeType fileName: nil! !

!WAAnchorTag methodsFor: '*seaside-session' stamp: 'pmm 11/14/2009 15:50'!
document: aDocument mimeType: mimeType fileName: fileName
	self url: (canvas context urlForDocument: aDocument mimeType: mimeType fileName: fileName).
	mimeType isNil ifFalse: [
		self type: mimeType ]! !

!WAIframeTag methodsFor: '*seaside-session' stamp: 'lr 8/9/2006 08:34'!
document: anObject
	self document: anObject mimeType: nil! !

!WAIframeTag methodsFor: '*seaside-session' stamp: 'lr 8/9/2006 08:31'!
document: anObject mimeType: aMimeString
	self document: anObject mimeType: aMimeString fileName: nil! !

!WAIframeTag methodsFor: '*seaside-session' stamp: 'lr 10/28/2007 14:42'!
document: anObject mimeType: aMimeString fileName: aFileNameString
	self url: (canvas context
		urlForDocument: anObject
		mimeType: aMimeString
		fileName: aFileNameString)! !

WARequestFilter subclass: #WAMutualExclusionFilter
	instanceVariableNames: 'mutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session-Filter'!

!WAMutualExclusionFilter methodsFor: 'processing' stamp: 'jf 2/5/2009 11:01'!
handleFiltered: aRequestContext
	(self shouldTerminate: aRequestContext)
		ifTrue: [ self terminate: aRequestContext ]
		ifFalse: [ mutex critical: [ super handleFiltered: aRequestContext ] ]! !

!WAMutualExclusionFilter methodsFor: 'initialization' stamp: 'jf 2/5/2009 10:54'!
initialize
	super initialize.
	mutex := WAMutex new! !

!WAMutualExclusionFilter methodsFor: 'private' stamp: 'jf 1/14/2009 13:34'!
shouldTerminate: aRequestContext
	^ (aRequestContext request fields includesKey: 'terminate')
		and: [ aRequestContext application preferenceAt: #sessionAllowTermination ]! !

!WAMutualExclusionFilter methodsFor: 'private' stamp: 'lr 5/20/2009 20:13'!
terminate: aRequestContext
	mutex terminateOwner.
	aRequestContext respond: [ :response |
		response
			contentType: WAMimeType textPlain;
			nextPutAll: 'Process terminated' ]! !

!WARequestFilter methodsFor: '*seaside-session' stamp: 'lr 8/31/2008 14:45'!
updateStates: aSnapshot
	aSnapshot register: next! !

!WAImageTag methodsFor: '*seaside-session' stamp: 'lr 6/20/2007 10:52'!
document: aDocument
	self document: aDocument mimeType: nil! !

!WAImageTag methodsFor: '*seaside-session' stamp: 'lr 6/20/2007 10:52'!
document: aDocument mimeType: mimeType
	self document: aDocument mimeType: mimeType fileName: nil! !

!WAImageTag methodsFor: '*seaside-session' stamp: 'jf 9/17/2009 11:54'!
document: aDocument mimeType: mimeType fileName: fileName
	self url: (canvas context urlForDocument: aDocument mimeType: mimeType fileName: fileName)! !

!GRPackage class methodsFor: '*seaside-session' stamp: 'lr 2/17/2010 15:13'!
seasideSession
	^ self new
		name: 'Seaside-Session';
		description: 'Support for stateful and secure session handling.';
		addDependency: 'Seaside-Core';
		url: #seasideUrl;
		yourself! !

!WARenderContext methodsFor: '*seaside-session' stamp: 'lr 10/28/2007 14:42'!
urlForDocument: anObject
	^ self urlForDocument: anObject mimeType: nil! !

!WARenderContext methodsFor: '*seaside-session' stamp: 'avi 5/22/2004 18:02'!
urlForDocument: anObject mimeType: mimeString
	^ self urlForDocument: anObject mimeType: mimeString fileName: nil! !

!WARenderContext methodsFor: '*seaside-session' stamp: 'pmm 2/18/2012 11:21'!
urlForDocument: anObject mimeType: mimeType fileName: fileName
	| handler |
	handler := (WADocumentHandler
		document: anObject
		mimeType: mimeType
		fileName: fileName).
	^ self requestContext session register: handler! !

WASystemConfiguration subclass: #WAApplicationConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WAApplicationConfiguration commentStamp: 'jf 7/22/2008 12:40' prior: 0!
WAApplicationConfiguration defines attributes required by WAApplication. All applications should include this configuration (this is done by default for new applications).

See #addAttributes: for details on provided attributes.!

!WAApplicationConfiguration methodsFor: 'description' stamp: 'jf 8/24/2011 11:48'!
continuationClasses
	^ (WASessionContinuation allSubclasses reject: [ :each | each isAbstract ])
		asSortedCollection: [ :a :b | a name <= b name ]! !

!WAApplicationConfiguration methodsFor: 'description' stamp: 'jf 3/13/2012 12:26'!
describeOn: config
	(config collection: #libraries)
		options: [ self libraryClasses ].
	(config list: #initialContinuationClass)
		beAdvanced;
		options: [ self continuationClasses ];
		default: WANullSessionContinuation.
	(config string: #mimeType)
		beAdvanced;
		label: 'MIME Type';
		comment: 'The media type of HTML documents reported to the browser.';
		default: 'text/html'.
	(config url: #resourceBaseUrl)
		group: #server;
		comment: 'Sets the base URL for URLS generated with #resourceUrl:'.
	(config boolean: #sessionAllowTermination)
		comment: 'Whether sessions allows a special HTTP request to terminate the process currently running in the session. This is used during development and should be disabled for deployment.';
		default: false.
	(config list: #sessionClass)
		comment: 'Sessions in this application will be instances of this class.';
		options: [ self sessionClasses ];
		default: WASession.
	(config listOfClasses: #trackingStrategy)
		comment: 'How sessions and document handlers should be tracked.';
		options: [ self trackingStrategyClasses ];
		default: WAQueryFieldHandlerTrackingStrategy new
! !

!WAApplicationConfiguration methodsFor: 'description' stamp: 'lr 1/10/2009 15:20'!
libraryClasses
	^ WAFileLibrary allSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

!WAApplicationConfiguration methodsFor: 'ancestry' stamp: 'pmm 7/24/2011 12:48'!
parents
	"Skip WARegistryConfiguration because we override #trackingStrategy. It doesn't work otherwise."
	^ Array with: WARequestHandlingConfiguration instance! !

!WAApplicationConfiguration methodsFor: 'description' stamp: 'jf 7/22/2008 12:00'!
sessionClasses
	^ WASession withAllSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

!WAApplicationConfiguration methodsFor: 'description' stamp: 'pmm 7/24/2011 13:34'!
trackingStrategyClasses
	^ (WAHandlerTrackingStrategy allSubclasses reject: [ :each | each isAbstract ])
		asSortedCollection: [ :a :b | a name <= b name ]! !

!WARequestHandler methodsFor: '*seaside-session' stamp: 'mb 5/26/2006 08:49'!
isApplication
	^ false! !

!WARequestHandler methodsFor: '*seaside-session' stamp: 'jf 7/25/2008 20:33'!
isSession
	^ false! !

WARequestHandler subclass: #WASession
	instanceVariableNames: 'continuations properties cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WASession commentStamp: 'pmm 2/18/2012 11:03' prior: 0!
I am a Seaside session. A new instance of me gets created when an user accesses an application for the first time and is persistent as long as the user is interacting with it.

This class is intended to be subclasses by applications that need global state, like a user. Custom state can be added by creating instance variables and storing it there. The session can be retrieved by #session if inside a component or task or by evaluating: WACurrentRequestContext session

If the session has not been used for #defaultTimeoutSeconds, it is garbage collected by the system. To manually expire a session call #expire.

A good way to clear all sessions is the following code:

WARegistry clearAllHandlers.
WAPlatform current garbageCollect!

!WASession methodsFor: 'accessing' stamp: 'lr 4/15/2007 15:55'!
actionField
	^ '_k'! !

!WASession methodsFor: 'accessing' stamp: 'jf 8/23/2011 14:09'!
actionUrlForContinuation: aContinuation
	^ self actionUrlForKey: (continuations store: (aContinuation setParent: self; yourself))! !

!WASession methodsFor: 'accessing' stamp: 'jf 2/15/2010 02:08'!
actionUrlForKey: aString
	^ self url
		addField: self actionField value: aString;
		yourself! !

!WASession methodsFor: 'accessing' stamp: 'jf 9/27/2009 23:59'!
application
	^ self parent! !

!WASession methodsFor: 'private' stamp: 'pmm 2/18/2012 11:04'!
buildCache
	^ WACache new! !

!WASession methodsFor: 'accessing' stamp: 'pmm 2/18/2012 11:05'!
cache
	cache isNil ifTrue: [
		cache := self buildCache ].
	^ cache! !

!WASession methodsFor: 'accessing' stamp: 'pmm 2/18/2012 13:30'!
cache: aCache
	cache := aCache! !

!WASession methodsFor: 'initialization' stamp: 'jf 1/10/2009 15:36'!
createCache
	"To configure the continuation cache you must currently subclass WASession and
	override this method."
	^ WACache new
		setExpiryPolicy: (WALeastRecentlyUsedExpiryPolicy new);
		yourself.! !

!WASession methodsFor: 'private' stamp: 'pmm 2/18/2012 11:08'!
documentHandlerFieldName
	^ '_d'! !

!WASession methodsFor: 'handling' stamp: 'pmm 2/18/2012 11:16'!
handleFiltered: aRequestContext
	| key continuation fields documentHandlerKey |
	fields := aRequestContext request fields.
	documentHandlerKey := fields at: self documentHandlerFieldName ifAbsent: [ nil ].
	documentHandlerKey isNil ifFalse: [
		| documentHandler |
		documentHandler := self cache at: documentHandlerKey ifAbsent: [ 
			^ aRequestContext responseGenerator
				notFound;
				respond ].
		^ documentHandler handle: aRequestContext ].
	
	key := fields
		at: self actionField
		ifAbsent: [ ^ self start ].
		
	continuation := continuations
		at: key
		ifAbsent: [ ^ self unknownRequest ].
	continuation handle: aRequestContext! !

!WASession methodsFor: 'initialization' stamp: 'jf 1/29/2009 19:54'!
initialize
	super initialize.
	self initializeFilters.
	continuations := self createCache! !

!WASession methodsFor: 'initialization' stamp: 'jf 3/4/2009 10:13'!
initializeFilters
	self addFilter: WAMutualExclusionFilter new! !

!WASession methodsFor: 'testing' stamp: 'jf 7/25/2008 20:33'!
isSession
	^ true! !

!WASession methodsFor: 'scripts' stamp: 'pmm 8/27/2011 17:31'!
jumpToAnchor: aString
	| actionContinuation |
	actionContinuation := self requestContext handlers
		detect: [ :each | each isActionContinuation ]
		ifNone: [ WAIllegalStateException signal: 'You can only send #jumpToAnchor: from within a callback' ].
	actionContinuation jumpToAnchor: aString! !

!WASession methodsFor: 'accessing' stamp: 'jf 10/27/2008 14:25'!
key
	^ self application keyFor: self! !

!WASession methodsFor: 'copying' stamp: 'jf 11/3/2008 08:57'!
postCopy
	super postCopy.
	continuations := continuations copy! !

!WASession methodsFor: 'accessing' stamp: 'jf 9/30/2009 00:26'!
properties
	"Answer a dictionary of session-related properties."
	^ properties ifNil: [ properties := GRSmallDictionary new ]! !

!WASession methodsFor: 'parent/child' stamp: 'pmm 2/18/2012 11:23'!
register: aRequestHandler
	| key |
	key := self cache store: (aRequestHandler setParent: self; yourself).
	^ self basicUrl
		addField: self documentHandlerFieldName value: key;
		yourself! !

!WASession methodsFor: 'handling' stamp: 'jf 8/24/2011 11:40'!
start
	self application initialContinuationClass new handle: self requestContext! !

!WASession methodsFor: 'handling' stamp: 'jf 10/19/2008 12:30'!
unknownRequest
	^ self start! !

!WASession methodsFor: 'updating' stamp: 'lr 9/8/2009 12:40'!
updateRoot: anHtmlRoot! !

!WASession methodsFor: 'updating' stamp: 'jf 3/4/2009 13:50'!
updateStates: aSnapshot
	self filters do: [ :each | each updateStates: aSnapshot ].
	aSnapshot register: filter! !

!WASession methodsFor: 'updating' stamp: 'lr 9/8/2009 12:50'!
updateUrl: aUrl! !

WARequestHandler subclass: #WASessionContinuation
	instanceVariableNames: 'states'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WASessionContinuation commentStamp: 'lr 6/2/2008 09:19' prior: 0!
I represent a continuation as part of the flow of pages within a session. I am not a real continuation (as compared to those used in #call: and #answer:), I only represent a specific point in the session. I reference the root component and a memory snapshot of backtracked objects.!

WASessionContinuation subclass: #WANullSessionContinuation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WANullSessionContinuation methodsFor: 'handling' stamp: 'jf 3/13/2012 12:26'!
handleFiltered: aRequestContext
	"Do nothing"! !

!WASessionContinuation class methodsFor: 'private' stamp: 'jf 8/23/2011 16:08'!
defaultSnapshot
	^ WASnapshot new! !

!WASessionContinuation class methodsFor: 'testing' stamp: 'jf 8/24/2011 11:32'!
isAbstract
	^ self name = #WASessionContinuation! !

!WASessionContinuation class methodsFor: 'instance creation' stamp: 'jf 8/23/2011 16:07'!
new
	^ self snapshot: self defaultSnapshot! !

!WASessionContinuation class methodsFor: 'instance creation' stamp: 'jf 8/23/2011 16:07'!
snapshot: aSnapshot
	^ self basicNew
		initializeWithSnapshot: aSnapshot;
		yourself! !

!WASessionContinuation methodsFor: 'processing' stamp: 'jf 1/21/2009 14:11'!
captureState
	"self states reset"
	"Setting states to nil matches the behaviour of Seaside 2.8. Resetting the Snapshot
	is useful for JS callbacks in that refreshing the page will show the modified state
	but can also result in many ActionPhaseContinuations all sharing the same state
	but with different callbacks, some of which may reference state that is no longer present."
	states := nil.
	self updateStates: self states! !

!WASessionContinuation methodsFor: 'evaluating' stamp: 'jf 8/23/2011 13:47'!
handle: aRequestContext
	"Resume processing of a request. To ensure valid application state restore all registered states."

	self states restore.
	self withUnregisteredHandlerDo: [ super handle: aRequestContext ]! !

!WASessionContinuation methodsFor: 'initialization' stamp: 'jf 8/23/2011 16:06'!
initializeWithSnapshot: aSnapshot
	self initialize.
	states := aSnapshot! !

!WASessionContinuation methodsFor: 'private' stamp: 'lr 1/19/2009 19:09'!
redirectToContinuation: aSessionContinuation
	self respond: [ :response | response redirectTo: aSessionContinuation registerForUrl ]! !

!WASessionContinuation methodsFor: 'private' stamp: 'lr 3/8/2009 10:55'!
registerForUrl
	^ self registerForUrl: true! !

!WASessionContinuation methodsFor: 'private' stamp: 'lr 3/8/2009 10:55'!
registerForUrl: aBoolean
	| url |
	url := self session actionUrlForContinuation: self.
	aBoolean ifTrue: [ self updateUrl: url ].
	^ url! !

!WASessionContinuation methodsFor: 'accessing' stamp: 'lr 4/9/2008 10:52'!
request
	"Answer the current request object."

	^ self requestContext request! !

!WASessionContinuation methodsFor: 'processing' stamp: 'lr 5/20/2009 20:14'!
respond: aBlock 
	self requestContext respond: aBlock! !

!WASessionContinuation methodsFor: 'accessing' stamp: 'lr 1/19/2009 23:00'!
states
	^ states ifNil: [ states := WASnapshot new ]! !

!WASessionContinuation methodsFor: 'updating' stamp: 'jf 9/28/2008 23:07'!
updateStates: aSnapshot
	self session updateStates: aSnapshot! !

!WASessionContinuation methodsFor: 'updating' stamp: 'lr 9/8/2009 12:52'!
updateUrl: aUrl
	self session updateUrl: aUrl! !

!WASessionContinuation methodsFor: 'private' stamp: 'jf 11/3/2009 01:11'!
withUnregisteredHandlerDo: aBlock 
	^ aBlock 
		on: WAUnregisteredHandlerError
		do: [ :err | self requestContext redirectTo: self application url ]! !

!WAContentElement methodsFor: '*seaside-session' stamp: 'lr 3/27/2007 13:32'!
document: aString
	self document: aString mimeType: self typeOrNil! !

!WAContentElement methodsFor: '*seaside-session' stamp: 'lr 3/27/2007 13:32'!
document: aString fileName: fileName
	self document: aString mimeType: self typeOrNil fileName: fileName! !

!WAContentElement methodsFor: '*seaside-session' stamp: 'lr 3/27/2007 13:32'!
document: aString mimeType: mimeType
	self document: aString mimeType: mimeType fileName: nil! !

!WAContentElement methodsFor: '*seaside-session' stamp: 'lr 3/27/2007 13:33'!
document: aString mimeType: mimeType fileName: fileName
	self url: (root context urlForDocument: aString mimeType: mimeType fileName: fileName)! !

WARegistry subclass: #WAApplication
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WAApplication commentStamp: 'rew 10/19/2007 10:47' prior: 0!
WAApplication is the starting point for a Seaside application. When a WAComponent is registered as a top level component a WAApplication object is added to a WADispatcher. The dispatcher forwards all requests to the WAApplication, which in turn forwards them to the correct WASession object. WAApplication's parent class WARegistry maintains a list of all active sessions to the application. 

"configuration" contains a chain of WAConfituration classes that define attributes of the application. The attribute "rootComponent", for example, defines the top level WAComponent class for the application. The configuration chain includes WAUserConfiguration, WAGlobalConfiguration, WARenderLoopConfiguration and WASessionConfiguration. Other configurations can be added to the chain when the top level application is registered with a dispatcher. (See below)

"libraries" is a collection of WALibrary classes, which are used to serve css, javascript and images used by the application. These may be in methods or in files. Sometimes these libraries are replaced by static files served by Apache. See WAFileLibrary class comment for more information.

Registering an Application.
	An application can be registered with a dispatcher by using the Seaside configuration page or via code. Below MyComponent is a subclass of WAComponent. The following registers the component as an application, gives some values to attributes (or preferences) and adds a library and a configuration. 

MyComponent class>>initialize
	"self initialize"
	| application |
	application := self registerAsApplication: 'sample'.
	application preferenceAt: #sessionClass put: Glorp.WAGlorpSession.
	application addLibrary: SampleLibrary.
	application configuration addAncestor: GlorpConfiguration new.
	application preferenceAt: #glorpDatabasePlatform put: Glorp.PostgreSQLPlatform.
	application preferenceAt: #databaseServer put: '127.0.0.1'.
	application preferenceAt: #databaseConnectString put: 'glorptests'.

MyComponent>>someInstanceMethod
	"example of how to access attributes (preferences)"
	self session application preferenceAt: #glorpDatabasePlatform!

!WAApplication class methodsFor: 'accessing' stamp: 'lr 11/16/2006 18:38'!
description
	^ 'Application'! !

!WAApplication methodsFor: 'libraries' stamp: 'lr 11/18/2007 15:24'!
addLibrary: aLibraryClass
	self preferenceAt: #libraries put: (self libraries copyWith: aLibraryClass)! !

!WAApplication methodsFor: 'configuration' stamp: 'pmm 5/26/2010 07:23'!
contentType
	^ self mimeType seasideMimeType
		charset: self requestContext charSet;
		yourself! !

!WAApplication methodsFor: 'configuration' stamp: 'jf 1/30/2009 17:25'!
defaultConfiguration
	^ super defaultConfiguration
		removeParent: WARegistryConfiguration instance;
		addParent: WAApplicationConfiguration instance;
		yourself! !

!WAApplication methodsFor: 'handling' stamp: 'jf 9/24/2009 00:12'!
handleDefault: aRequestContext
	self handle: aRequestContext registering: self newSession
		! !

!WAApplication methodsFor: 'handling' stamp: 'pmm 7/16/2011 16:02'!
handleDefault: aRequestContext withKey: aKey
	self handle: aRequestContext registering: self newSession at: aKey
		! !

!WAApplication methodsFor: 'handling' stamp: 'pmm 6/20/2011 21:01'!
handleFiltered: aRequestContext 
	"Seaside applications do not respond to prefetch requests and only understand GET and POST requests."
	
	aRequestContext request isPrefetch 
		ifTrue: [ self handlePrefetch: aRequestContext ]
		ifFalse: [
			(self isImplemented: aRequestContext request method)
				ifTrue: [ super handleFiltered: aRequestContext ]
				ifFalse: [
					aRequestContext responseGenerator
						notImplemented;
						respond ] ]! !

!WAApplication methodsFor: 'private' stamp: 'pmm 6/20/2011 21:13'!
handlePrefetch: aRequestContext
	"Mozilla prefetches some <link> elements:
	https://developer.mozilla.org/en/Link_prefetching_FAQ
	This usally isn't a problem as long as they are used for style sheets.
	It is however a problem when they are used for navigational 'Index', 'Next', and 'Prev'
	See the following bug:
	http://code.google.com/p/pier/issues/detail?id=132
	
	Disabling has the disadvantage that we also disable prefech of things that would support prefetching
	(like style sheets). However we don't recommend to have Seaside serving these in production.
	
	We are not actually sure whether 403 Forbidden is the correct way to indicate that we don't support prefetching.
	Ohter options would include:
		- 412 Precondition Failed, user agent has to try different headers (seems fitting, however seems to be also about entity header)
		- 405 Method Not Allowed, but this is about HTTP methods and we must send Allow containg the valid methods
		
	We don't go through the response generator because the user will never see the response
	so it doesn't make sense to customize the contents."
	aRequestContext respond: [ :response |
		response forbidden ]! !

!WAApplication methodsFor: 'configuration' stamp: 'jf 8/24/2011 11:40'!
initialContinuationClass
	^ self preferenceAt: #initialContinuationClass! !

!WAApplication methodsFor: 'testing' stamp: 'lr 7/25/2011 19:57'!
isApplication
	^ true! !

!WAApplication methodsFor: 'private' stamp: 'lr 1/12/2009 08:40'!
isImplemented: aString
	"Answer whether the receiver implements the given HTTP method."

	^ #('GET' 'POST') includes: aString! !

!WAApplication methodsFor: 'configuration' stamp: 'lr 2/10/2008 15:59'!
libraries
	^ self preferenceAt: #libraries! !

!WAApplication methodsFor: 'configuration' stamp: 'pmm 4/6/2008 15:11'!
mimeType
	^ self preferenceAt: #mimeType! !

!WAApplication methodsFor: 'private' stamp: 'jf 9/24/2009 00:17'!
newSession
	"Create a new session which inherits our configuration."
	^ self sessionClass new
		configuration: (WAUserConfiguration new addParent: self configuration; yourself);
		yourself! !

!WAApplication methodsFor: 'libraries' stamp: 'lr 11/18/2007 15:25'!
removeLibrary: aLibraryClass
	self preferenceAt: #libraries put: (self libraries copyWithout: aLibraryClass)! !

!WAApplication methodsFor: 'configuration' stamp: 'lr 2/12/2007 22:02'!
resourceBaseUrl
	^ self preferenceAt: #resourceBaseUrl! !

!WAApplication methodsFor: 'configuration' stamp: 'jf 1/30/2009 17:21'!
sessionAllowTermination
	^ self preferenceAt: #sessionAllowTermination! !

!WAApplication methodsFor: 'configuration' stamp: 'lr 10/28/2007 14:42'!
sessionClass
	^ self preferenceAt: #sessionClass! !

!WAApplication methodsFor: 'accessing' stamp: 'lr 7/25/2011 19:03'!
sessions
	| collection |
	collection := OrderedCollection new.
	self sessionsDo: [ :session | collection add: session ].
	^ collection! !

!WAApplication methodsFor: 'enumerating' stamp: 'pmm 2/18/2012 12:54'!
sessionsDo: aBlock
	self handlersDo: aBlock! !

!WAImageButtonTag methodsFor: '*seaside-session' stamp: 'jf 9/17/2009 11:55'!
document: aDocument
	self document: aDocument mimeType: nil! !

!WAImageButtonTag methodsFor: '*seaside-session' stamp: 'jf 9/17/2009 11:55'!
document: aDocument mimeType: mimeType
	self document: aDocument mimeType: mimeType fileName: nil! !

!WAImageButtonTag methodsFor: '*seaside-session' stamp: 'jf 9/17/2009 11:54'!
document: aDocument mimeType: mimeType fileName: fileName
	self url: (canvas context urlForDocument: aDocument mimeType: mimeType fileName: fileName)! !

!WAObject methodsFor: '*seaside-session' stamp: 'lr 7/25/2011 19:53'!
application
	"Answer the current seaside application, instance of WAApplication or a subclass."
	
	^ self requestContext application! !

!WAObject methodsFor: '*seaside-session' stamp: 'lr 7/25/2011 19:53'!
session
	"Answer the current seaside session, instance of WASesssion or a subclass."
	
	^ self requestContext session! !

WAHandlerTrackingStrategy subclass: #WASessionTrackingStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WASessionTrackingStrategy commentStamp: 'pmm 7/17/2011 17:53' prior: 0!
I am the abstract base class for tracking strategies that track sessions differently from other request handlers (mostly WADocumentHandlers).

Concrete implementsions have to take care that expired document handlers don't expire sessions.!

WASessionTrackingStrategy subclass: #WACookieForBrowserIPForCrawlerTrackingStrategy
	instanceVariableNames: 'cookieStrategy ipStrategy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WACookieForBrowserIPForCrawlerTrackingStrategy commentStamp: 'pmm 7/24/2011 13:57' prior: 0!
WACookieForBrowserIPForCrawlerTrackingStrategy uses cookie based session tracking for browsers and IPs for crawlers (they don't support cookies). This way there is never a session key in any URL.

This is inspired by:
http://www.tomcatexpert.com/blog/2011/05/18/crawler-session-manager-valve!

!WACookieForBrowserIPForCrawlerTrackingStrategy methodsFor: 'public' stamp: 'pmm 2/18/2012 13:46'!
addKey: aString toContext: aRequestContext
	^ (self strategyFor: aRequestContext)
		addKey: aString toContext: aRequestContext! !

!WACookieForBrowserIPForCrawlerTrackingStrategy methodsFor: 'public' stamp: 'pmm 2/18/2012 13:48'!
addKey: aString toUrl: aUrl
	^ (self strategyFor: WACurrentRequestContext value)
		addKey: aString toUrl: aUrl! !

!WACookieForBrowserIPForCrawlerTrackingStrategy methodsFor: 'private' stamp: 'pmm 7/24/2011 13:57'!
crawlerSubStrings
	^ #(
		'bot'
		'Bot'
		'Yahoo!! Slurp'
		'Feedfetcher-Google'
	)! !

!WACookieForBrowserIPForCrawlerTrackingStrategy methodsFor: 'initialization' stamp: 'pmm 7/24/2011 14:04'!
initialize
	super initialize.
	ipStrategy := WAIPSessionTrackingStrategy new.
	cookieStrategy := WACookieOnlySessionTrackingStrategy new! !

!WACookieForBrowserIPForCrawlerTrackingStrategy methodsFor: 'private' stamp: 'pmm 7/24/2011 14:00'!
isFromCrawler: aRequestContext
	| userAgent |
	userAgent := aRequestContext request userAgent.
	^ userAgent notNil and: [ self crawlerSubStrings anySatisfy: [ :each |
			(userAgent indexOfSubCollection: each) > 0 ] ]! !

!WACookieForBrowserIPForCrawlerTrackingStrategy methodsFor: 'public' stamp: 'pmm 7/24/2011 14:05'!
keyFromContext: aRequestContext
	^ (self strategyFor: aRequestContext)
		keyFromContext: aRequestContext! !

!WACookieForBrowserIPForCrawlerTrackingStrategy methodsFor: 'public' stamp: 'pmm 7/24/2011 14:05'!
noHandlerFoundForKey: aString in: anApplication context: aRequestContext
	^ (self strategyFor: aRequestContext)
		noHandlerFoundForKey: aString in: anApplication context: aRequestContext! !

!WACookieForBrowserIPForCrawlerTrackingStrategy methodsFor: 'private' stamp: 'pmm 7/24/2011 14:03'!
strategyFor: aRequestContext
	^ (self isFromCrawler: aRequestContext)
		ifTrue: [ ipStrategy ]
		ifFalse: [ cookieStrategy ]! !

WASessionTrackingStrategy subclass: #WACookieSessionTrackingStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WACookieSessionTrackingStrategy commentStamp: 'pmm 7/17/2011 17:52' prior: 0!
I am the abstract base class for tracking strategies that use cookies to track sessions and query fields to track everything else.!

WACookieSessionTrackingStrategy subclass: #WACookieIfSupportedSessionTrackingStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WACookieIfSupportedSessionTrackingStrategy commentStamp: 'pmm 8/18/2011 09:14' prior: 0!
I use cookies to track sessions if the browser supports it and use query fields (?_s=key) otherwise. I keep adding the session cookie and the session query field until the session cookie shows up in the request. Then I stop adding the query field.!

!WACookieIfSupportedSessionTrackingStrategy methodsFor: 'public' stamp: 'pmm 2/18/2012 13:48'!
addKey: aString toUrl: aUrl
	"Only add a query field if there isn't already a cookie"
	self
		cookieFromContext: self requestContext
		ifAbsent: [ self addQueryFieldWithValue: aString to: aUrl ]! !

!WACookieIfSupportedSessionTrackingStrategy methodsFor: 'public' stamp: 'pmm 7/17/2011 12:39'!
noHandlerFoundForKey: aString in: anApplication context: aRequestContext
	| queryFieldKey |
	queryFieldKey := self queryFieldKeyFromContext: aRequestContext.
	queryFieldKey isNil
		ifTrue: [
			| cookie |
			"there is no query field -> a cookie was used for the look up -> delete it"
			cookie := self cookieFromContext: aRequestContext ifAbsent: [ nil ].
			cookie notNil ifTrue: [
				"If there is a cookie delete it.
				Unfortunately if the browser doesn't support cookies then an expired document handler will result in the session cookie being deleted."
				self deleteSessionCookieOf: anApplication from: aRequestContext ] ]
		ifFalse: [
			| cookie |
			cookie := self cookieFromContext: aRequestContext ifAbsent: [ nil ].
			(cookie notNil and: [ cookie value = queryFieldKey ]) ifTrue: [
				"A query field was used for the look up but there is also a cookie with the same value -> delete it"
				self deleteSessionCookieOf: anApplication from: aRequestContext ] ].
	super noHandlerFoundForKey: aString in: anApplication context: aRequestContext! !

WACookieSessionTrackingStrategy subclass: #WACookieOnlySessionTrackingStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WACookieOnlySessionTrackingStrategy commentStamp: 'pmm 7/17/2011 18:13' prior: 0!
I track sessions using cookies and everything else using query fields. This works fine as long as the client accepts cookies. This breaks for web crawlers (they don't accept cookies).!

!WACookieOnlySessionTrackingStrategy class methodsFor: 'testing' stamp: 'pmm 7/10/2011 23:36'!
isAbstract
	^ self name = #WACookieHandlerTrackingStragety! !

!WACookieOnlySessionTrackingStrategy methodsFor: 'public' stamp: 'pmm 2/18/2012 13:48'!
addKey: aString toUrl: aUrl
	"We never add a session key to the URL, the user agent has to support cookies"! !

!WACookieOnlySessionTrackingStrategy methodsFor: 'public' stamp: 'pmm 7/16/2011 17:02'!
noHandlerFoundForKey: aString in: anApplication context: aRequestContext
	| queryField |
	queryField := self queryFieldKeyFromContext: aRequestContext.
	aString = queryField ifFalse: [
		"Don't delete the session cookie if we have an expired document halder.
		Only delete the session cookie if we actually have an expired session."
		self deleteSessionCookieOf: anApplication from: aRequestContext ].
	super noHandlerFoundForKey: aString in: anApplication context: aRequestContext! !

!WACookieSessionTrackingStrategy class methodsFor: 'testing' stamp: 'pmm 7/16/2011 14:53'!
isAbstract
	^ self name = #WACookieSessionTrackingStrategy! !

!WACookieSessionTrackingStrategy methodsFor: 'private' stamp: 'pmm 7/10/2011 23:37'!
addCookieWithValue: aString to: aRequestContext
	"Create a new handler key cookie and att it to the response."
	
	aRequestContext response addCookie: 
		(aRequestContext newCookie
			key: self keyCookieName;
			value: aString;
			discard: true;
			comment: 'the Seaside Session Cookie';
			commentUrl: 'http://www.seaside.st/documentation/sessionCookie';
			yourself)! !

!WACookieSessionTrackingStrategy methodsFor: 'public' stamp: 'pmm 2/18/2012 13:46'!
addKey: aString toContext: aRequestContext
	| oldCookie |
	oldCookie := self cookieFromContext: aRequestContext ifAbsent: [ nil ].
	(oldCookie isNil or: [ oldCookie value ~= aString ]) ifTrue: [
		"only add the cookie if we don't already have one or it has the wrong key"
		self addCookieWithValue: aString to: aRequestContext ]! !

!WACookieSessionTrackingStrategy methodsFor: 'private' stamp: 'pmm 7/15/2011 21:20'!
cookieFromContext: aRequestContext ifAbsent: absentBlock
	"Answer the handler key cookie. If none is found, return the result of evaluating absentBlock.
	There can be several cookies with the correct key so we try to find the one with the most specific path."

	| candidates requestUrl |
	candidates := aRequestContext request cookiesAt: self keyCookieName.
	
	candidates isEmptyOrNil
		ifTrue: [ ^ absentBlock value ].
	candidates size = 1 
		ifTrue: [ ^ candidates first ].
		
	requestUrl := aRequestContext request url.
	candidates := candidates collect: [ :each |
		each -> (each path seasideUrl relativeTo: requestUrl) ].
	
	candidates := candidates reject: [ :each |
		| path |
		path := each value.
		path notEmpty and: [ path first = '..' ] ].

	candidates isEmpty
		ifTrue: [ ^ absentBlock value ].
		
	candidates size > 1 ifTrue: [
		candidates sort: [ :a :b |
			 a value size <= b value size ] ].
	
	^ candidates first key! !

!WACookieSessionTrackingStrategy methodsFor: 'private' stamp: 'pmm 7/17/2011 11:28'!
deleteSessionCookieOf: anApplication from: aRequestContext
	| cookie baseUrl |
	cookie := self cookieFromContext: aRequestContext ifAbsent: [ ^ self ].
	baseUrl := anApplication url.
	aRequestContext response 
		deleteCookie: cookie
		path:  baseUrl pathString
		domain: baseUrl host! !

!WACookieSessionTrackingStrategy methodsFor: 'private' stamp: 'pmm 7/15/2011 21:07'!
keyCookieName
	"Answer the name of the cookie to track the handler."
	
	^ self keyField! !

!WACookieSessionTrackingStrategy methodsFor: 'public' stamp: 'pmm 7/16/2011 15:45'!
keyFromContext: aRequestContext
	"If a query field is present it overrides a cookie.
	In the case of a document handler both a cookie and a query field are present with different values
	(the cookie 'pointing' to the session and the query field 'pointing' to the document handler).
	We have to take the document handler in this case."
	| key |
	key := self queryFieldKeyFromContext: aRequestContext.
	^ key isNil
		ifTrue: [ (self cookieFromContext: aRequestContext ifAbsent: [ ^ nil ]) value ]
		ifFalse: [ key ]! !

WASessionTrackingStrategy subclass: #WAInvisibleSessionTrackingStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WAInvisibleSessionTrackingStrategy commentStamp: 'pmm 8/14/2011 11:07' prior: 0!
I am the abstract base class for classes that that use some request attribute that is present on every request for tracking sessions. For document handlers query fields are used.!

WAInvisibleSessionTrackingStrategy subclass: #WAIPSessionTrackingStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WAIPSessionTrackingStrategy commentStamp: 'pmm 7/17/2011 17:38' prior: 0!
I track sessions using the remote address of a client. Usually you don't want to use this because it causes troubles when multiple users share the same IP (eg. several users from the same company). That's why I'm marked as abstract.

There are some special cases like crawlers where it can work though.!

!WAIPSessionTrackingStrategy class methodsFor: 'testing' stamp: 'pmm 7/16/2011 10:18'!
isAbstract
	^ true! !

!WAIPSessionTrackingStrategy methodsFor: 'private' stamp: 'pmm 7/16/2011 15:28'!
remoteAddressFromRequest: aRequest
	^ aRequest
			headerAt: 'x-forwarded-for'
			ifAbsent: [ aRequest remoteAddress ]! !

!WAIPSessionTrackingStrategy methodsFor: 'private' stamp: 'pmm 8/14/2011 11:11'!
sessionIdFromContext: aRequestContext
	^ self remoteAddressFromRequest: aRequestContext request! !

!WAInvisibleSessionTrackingStrategy class methodsFor: 'testing' stamp: 'pmm 8/14/2011 11:05'!
isAbstract
	^ self name = #WAInvisibleSessionTrackingStrategy! !

!WAInvisibleSessionTrackingStrategy methodsFor: 'public' stamp: 'pmm 2/18/2012 13:46'!
addKey: aString toContext: aRequestContext
	"The session id is already present, no need to add it."! !

!WAInvisibleSessionTrackingStrategy methodsFor: 'public' stamp: 'pmm 2/18/2012 13:47'!
addKey: aString toUrl: aUrl
	"The session id is already present, no need to add it."! !

!WAInvisibleSessionTrackingStrategy methodsFor: 'public' stamp: 'pmm 8/14/2011 11:09'!
keyFromContext: aRequestContext
	"If a query field is present it overrides a session id.
	In the case of a document handler both a session id and a query field are present with different values
	(the session id 'pointing' to the session and the query field 'pointing' to the document handler).
	We have to take the document handler in this case."
	| key |
	key := self queryFieldKeyFromContext: aRequestContext.
	^ key isNil
		ifTrue: [ self sessionIdFromContext: aRequestContext ]
		ifFalse: [ key ]! !

!WAInvisibleSessionTrackingStrategy methodsFor: 'public' stamp: 'pmm 8/14/2011 11:04'!
noHandlerFoundForKey: aString in: anApplication context: aRequestContext
	aString = (self keyFromContext: aRequestContext)
		ifTrue: [
			"expired session, create a new session"
			anApplication handleDefault: aRequestContext withKey: aString ]
		ifFalse: [
			"repired document handler, answer not found"
			super noHandlerFoundForKey: aString in: anApplication context: aRequestContext ]! !

!WAInvisibleSessionTrackingStrategy methodsFor: 'private' stamp: 'pmm 8/14/2011 11:09'!
sessionIdFromContext: aRequestContext
	 self subclassResponsibility! !

WAInvisibleSessionTrackingStrategy subclass: #WASslSessionTrackingStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Session'!

!WASslSessionTrackingStrategy commentStamp: 'pmm 8/14/2011 12:41' prior: 0!
I track sessions using the SSL session id. This has the advantage that the session id never shows up in the request as a parameter or cookie.

Special server configuration to set the life time of an SSL session may be required.

http://www.roseindia.net/javacertification/ibm-287/http_sessions.shtml
http://publib.boulder.ibm.com/infocenter/iseries/v5r4/index.jsp?topic=%2Frzatz%2F51%2Fprogram%2Fsesdeci.htm
http://en.wikipedia.org/wiki/Session_fixation#Solution:_Utilize_SSL_.2F_TLS_Session_identifier!

!WASslSessionTrackingStrategy methodsFor: 'public' stamp: 'pmm 8/14/2011 11:11'!
sessionIdFromContext: aRequestContext
	^ aRequestContext request sslSessionId! !

!WASessionTrackingStrategy class methodsFor: 'testing' stamp: 'pmm 7/16/2011 14:52'!
isAbstract
	^ self name = #WASessionTrackingStrategy! !
